// API: https://airvpn.org/forums/topic/14737-api/
const { error } = require('console');
const crypto = require('crypto');

class ipleaktest {

  sessionHash;
  randomCharsForEveryDNSRequest;
  DNSqueryURL;
  IPInfoURL = "https://ipv4.ipleak.net/json/";

  constructor() {
    this.sessionHash = crypto.randomBytes(20).toString('hex');
    this.randomCharsForEveryDNSRequest = null;
    this.DNSqueryURL = null;
  }

  async RequestHostIpInfo() {
    let result = await fetch(this.IPInfoURL);
    if (!result) { return new Error("Error: Unable to fetch IP information");}
    let parsedData = await result.json();

    // let parsedData = this.#createTestData();
    return parsedData.isp_name + 
      ' | ' + parsedData.continent_name +
      ' | ' + parsedData.country_name +
      ' | ' + parsedData.ip;
  }

  async RequestISP(ip){
    let query = this.IPInfoURL.concat(ip);
    let result = await fetch(query);
    if (!result) { return new Error("Error: Unable to fetch IP information");}
    let parsedData = await result.json();
    let isp = new Object();
    isp.isp_name = parsedData.isp_name;
    isp.country_name = parsedData.country_name;
    isp.city_name = parsedData.city_name;
    return isp;
  }

  async RequestDNS(){
    // Initiate values     
    this.randomCharsForEveryDNSRequest = crypto.randomBytes(3).toString('hex');
    this.DNSqueryURL = `https://${this.sessionHash}-${this.randomCharsForEveryDNSRequest}ipv4.ipleak.net/dnsdetection/`

    // Query DNS
    let queryResult = await fetch(this.DNSqueryURL);
    if (!queryResult) { return new Error("Error: Unable to perform DNS query");}

    // Work on the data
    let parsedData = await queryResult.json();
    let collectedProviders = [];
    let dnsProviders = [];

    for (var i=0, n=Object.keys(parsedData.ip).length; i < n; ++i ) {
        collectedProviders.push(this.RequestISP(Object.keys(parsedData.ip)[i]));
    }

    // Resolve all collected dns providers
    const resolvedDNSProviders = await Promise.all(collectedProviders);

    // Loop through all providers and map them with their respective IP
    resolvedDNSProviders.forEach((provider, i) => {
    let dns = new Object();
    dns.ip = Object.keys(parsedData.ip)[i];
    dns.dns_name = provider.isp_name;
    dns.country_name = provider.country_name;
    dns.city_name = provider.city_name;
    dnsProviders.push(dns);
  });

    return dnsProviders;
  }

  #createTestData(){
    let testData = {
      as_number: 45454,
      isp_name: 'TEST ISP',
      country_code: 'US',
      country_name: 'United States',
      region_code: 'ZL',
      region_name: 'LA County',
      continent_code: 'NA',
      continent_name: 'North America',
      city_name: 'CA',
      postal_code: null,
      postal_confidence: null,
      latitude: 0,
      longitude: 0,
      accuracy_radius: 10,
      time_zone: '??',
      metro_code: null,
      level: 'min',
      cache: 1736285403,
      ip: '1.1.1.1',
      reverse: '',
      query_text: '1.2.3.4',
      query_type: 'myip',
      query_date: 1736285403
    };

    return testData;
  }

};

module.exports = {ipleaktest}
